#Copyright Massachusetts Institute of technology, 2003.
#Written by Eric Prud'hommeaux for the World Wide Web Consortium

# W3C::Rdf::Algae2 -- a parser/evaluator for the algae2 language
# 
# $Id: Algae2.pm,v 1.40 2004/06/13 07:08:42 eric Exp $
# 
use strict;
require Exporter;

package W3C::Rdf::Algae2;
use vars qw(@ISA @EXPORT $QL_ALGAE $QL_RDQL $QL_SeRQL);
@ISA = qw(W3C::Util::NamedParmObject Exporter);
@EXPORT = qw($QL_ALGAE $QL_RDQL $QL_SeRQL) ; # &getTripleAttribute &setTripleAttribute);
($QL_ALGAE, $QL_RDQL, $QL_SeRQL) = ('algae', 'RDQL', 'SeRQL');

use W3C::Rdf::Atoms; # qw($ATTRIB_GroundFact);
# Access to special variables
sub getTripleAttribute { # static
    my ($varname, $triple) = @_;
    if ($varname eq 'ENCODING') {
	my $o = $triple->getObject;
	return $o->isa('W3C::Rdf::String') ? $o->getEncoding : undef;
    } elsif ($varname eq 'DATATYPE') {
	my $o = $triple->getObject;
	return $o->isa('W3C::Rdf::String') ? $o->getDatatype : undef;
    } elsif ($varname eq 'LANG') {
	my $o = $triple->getObject;
	return $o->isa('W3C::Rdf::String') ? $o->getLang : undef;
    } elsif ($varname eq 'ATTRIB') {
	return $triple->getAttribution->getUri;
    } else {
	&throw(new W3C::Util::Exception(-message =>"unknown key word \"$varname\""));
    }
}
sub setTripleAttribute { # static
    my ($varname, $triple, $value, $atomDictionary) = @_;
    if ($varname eq 'ENCODING') {
	$triple->getObject->setEncoding($value);
    } elsif ($varname eq 'DATATYPE') {
	$triple->getObject->setDatatype($value);
    } elsif ($varname eq 'LANG') {
	$triple->getObject->setLang($value);
    } elsif ($varname eq 'ATTRIB') {
	my $auth = undef; # !!!
	$triple->setAttribution($atomDictionary->getAttribution($ATTRIB_GroundFact, 
									 $value, $auth, 
									 undef, 1, undef, 0));
    } else {
	&throw(new W3C::Util::Exception(-message =>"unknown key word \"$varname\""));
    }
    return $value;
}

use W3C::Util::Exception;
use W3C::Rdf::AlgaeParser;
use W3C::Rdf::RdqlParser;
use W3C::Rdf::SeRQLParser;
use W3C::Rdf::ResultSet;

# <ACCESSORS>
sub new {
    my ($proto, $atomDictionary, $namespaceHandler, $sources, $rdfApp, $sourceAttrib, $flags, @parms) = @_;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new(@parms);
    bless ($self, $class);

    $self->{-atomDictionary} = $atomDictionary;
    if (!$self->{-atomDictionary}) {
	&throw(new W3C::Util::Exception(-message => "$class must have an -atomDictionary"));
    }
    if (!UNIVERSAL::can($self->{-atomDictionary}, 'getUriLink')) {
	&throw(new W3C::Util::Exception(-message => "$class must have an -atomDictionary (not $self->{-atomDictionary})"));
    }

    $self->{-sourceAttribution} = $sourceAttrib;
    $self->{FLAGS} = $flags;
    $self->{-namespaceHandler} = $namespaceHandler;
    $self->{-sources} = $sources;
    $self->{DEFAULT_PARMS} = {};
    $self->{-rdfApp} = $rdfApp;
    $self->{-errorHandler} = undef; # $errorHandler;
    $self->{RESULT_SET} = new W3C::Rdf::ResultSet(-atomDictionary => $self->{-atomDictionary});

    # Report storage
    $self->{LABELS} = [];
#    $self->{COLLECTS} = [];
    $self->{HOST_symbols} = {};
    $self->{HOST_cache} = {};
    $self->{HOST_index} = 0;

    $self->{NOW_LOADING} = {};		# efficiently detect circular dependecny
    $self->{LOADING_STACK} = [];	# display circular dependecny route
    return $self;
}
sub include {
    my ($self, $src, $debug) = @_;
    my $url = $src->getUrl;
    if ($self->{NOW_LOADING}{$url}) {
	&throw(new W3C::Util::Exception(-message => "circular dependency on $url: ".join('->,', @{$self->{LOAD_STACK}})));
    }
    require W3C::XML::InputSource;
    my $inp = new W3C::XML::URIInputSource($url, {}); # No parms yet
    my $algaeString = $inp->getByteStream;
    my $p = new W3C::Rdf::SimpleAlgaeParser($algaeString, $self, $url);
    $self->{NOW_LOADING}{$url} = $p;
    push (@{$self->{LOAD_STACK}}, $url);
    my $actions = $p->parse($debug);
    pop (@{$self->{LOAD_STACK}});
    delete $self->{NOW_LOADING}{$url};
    return @$actions;
}
sub interpret {
    my ($self, $algaeString, $location, $lang, $debug) = @_;
    my $p = $lang eq $QL_ALGAE ? new W3C::Rdf::SimpleAlgaeParser($algaeString, $self, $location) : 
	$lang eq $QL_RDQL ? new W3C::Rdf::SimpleRdqlParser($algaeString, $self, $location) : 
	$lang eq $QL_SeRQL ? new W3C::Rdf::SimpleSeRQLParser($algaeString, $self, $location) : 
	&throw(new W3C::Util::Exception(-message => "unknown query language \"$lang\""));
    my $actions = $p->parse($debug);

    # print '('.&W3C::Rdf::AlgaeCompileTree::format(join ("\n", map {$_->toString} @$actions), ' ').')';

    foreach my $action (@$actions) {
	# print join("\n", $action->toString()),"\n";
	$action->delayedEvaluate($self->{RESULT_SET}, {}, undef, $self->getSourceAttribution);
    }

    #print $commandSet->toString,"\n";
    #print $commandSet->{RESULT_SET}->toString({-firstTitle => "interpretAlgae"}), "\n";
    return $self->getReport();
}
sub toString {
    my ($self, $precTable, $parentPrec) = @_;
    return "$self: FLAGS: ".join(' ', @{$self->{FLAGS}})." resultSet\n".$self->{RESULT_SET}->toString({-firstTitle => "interpretAlgae"})."\n";
}

# Cute whistle to dump the query to a file as toDot or toString format.
sub dumpStuff {
    my ($self, $category, $serializer, @args) = @_;
    if (exists $self->{DEFAULT_PARMS}{"dump${category}Mode"}) {
	my $outUrlStr = '-'; # file:///dev/tty
	if (exists $self->{DEFAULT_PARMS}{"dump${category}URL"}) {
	    $outUrlStr = $self->{DEFAULT_PARMS}{"dump${category}URL"}->getUrl;
	}
	my $outUrl = URI::new('URI', $outUrlStr);
	if ($outUrl->scheme eq 'file' && ! -e $outUrl->file && ! open(TMP, '>', $outUrl->file)) {
	    &throw(new W3C::Util::FileCreationException(-filename => $outUrl->file));
	}
	my $outHandle = &W3C::Rdf::RdfApp::getInputSource($outUrlStr, {});
	if ($self->{DEFAULT_PARMS}{"dump${category}Mode"}->getUrl eq 'http://www.w3.org/1999/02/26-modules/algae#DOT') {
	    $outHandle->setByteStream($serializer->serializeDot($category, @args), undef);
	} else {
	    $outHandle->setByteStream($serializer->serializeString($category, @args)."\n", undef);
	}
    }

}
sub serializeString {
    my ($self, $category, $expr, $precTable, $parentPrec) = @_;
    return $expr->toString($precTable, $parentPrec);
}
sub serializeDot {
    my ($self, $category, $expr, $precTable, $parentPrec) = @_;
    my $dot = $category eq 'Query' ? $expr->toDot : $expr->toGraph;
    return "digraph dotfile{ 
node [fontname=arial,fontsize=10,color=Black,fontcolor=Blue];
edge [fontname=arial,fontsize=10,color=Darkgreen,fontcolor=Red];
rankdir=TB;

$dot
}

";
}
# </ACCESSORS>

# <ACTIONs>
sub ask {
    my ($self, $expr, $optDbSpec) = @_;
    my $db = $optDbSpec ? $optDbSpec : $self->getDefaultDB;
    my $ret = new W3C::Rdf::AlgaeCompileTree::Ask($expr, $db, $self->{-defaultDB}, $self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}

sub assert {
    my ($self, $expr, $optDbSpec) = @_;
    my $db = $optDbSpec ? $optDbSpec : $self->getDefaultDB;
    my $ret = new W3C::Rdf::AlgaeCompileTree::Assert($expr, $db, $self->{-defaultDB}, $self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}

sub attach {
    my ($self, $type, $name, $bindings) = @_;
    my $ret = new W3C::Rdf::AlgaeCompileTree::Attach($type, $name, $self->_getParms($bindings), $self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}

sub clear {
    my ($self) = @_;
    my $ret = new W3C::Rdf::AlgaeCompileTree::Clear($self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}

sub collect {
    my ($self, $nodes) = @_;
    my $ret = new W3C::Rdf::AlgaeCompileTree::Collect($nodes, $self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}

sub collectStar {
    my ($self) = @_;
    my $ret = new W3C::Rdf::AlgaeCompileTree::CollectStar($self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}

sub host {
    my ($self, $nodes) = @_;
    my $ret = new W3C::Rdf::AlgaeCompileTree::Host($nodes, $self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}

sub flags {
    my ($self, $bindings, $setOp) = @_;
    my $ret = new W3C::Rdf::AlgaeCompileTree::Flags($bindings, $setOp, $self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}

sub fwrule {
    my ($self, $body, $head) = @_;
    my $ret = new W3C::Rdf::AlgaeCompileTree::FwRule($body, $head, $self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}

sub namespace {
    my ($self, $prefix, $ns) = @_;
    my $ret = new W3C::Rdf::AlgaeCompileTree::NamespaceDecl($prefix, $ns->getUrl, $self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}

sub slurp {
    my ($self, $src, $bindings, $optDbSpec) = @_;
    my $db = $optDbSpec ? $optDbSpec : $self->getDefaultDB;
    my $ret = new W3C::Rdf::AlgaeCompileTree::Slurp($src, $self->_getParms($bindings), $db, $self->{-rdfApp}, $self);
    $ret->immediateEvaluate($self->{RESULT_SET});
    return $ret;
}
sub _getParms {
    my ($self, $bindings) = @_;
    my $parms = {-atomDictionary => $self->{-atomDictionary}, 
		 -errorHandler => $self->{-errorHandler}, 
		 -lazyReification => 1, # some defaults
		 %{$self->{DEFAULT_PARMS}}};

    foreach my $binding (@$bindings) {
	my ($attr, $value) = ($binding->{NAME}, $binding->{VALUE});
	$parms->{'-'.$attr} = $value->val;
    }
    return $parms;
}
# </ACTIONs>

# <ACTION Acessors>
sub getResultSet {$_[0]->{RESULT_SET}}

#sub getCollects {$_[0]->{COLLECTS}}
sub getHostSymbols {$_[0]->{HOST_symbols}}
sub getDefaultParms {$_[0]->{DEFAULT_PARMS}}
sub getLabels {$_[0]->{LABELS}}
sub addLabel {
    my ($self, $label, $value) = @_;
    push (@{$self->{LABELS}}, $label);
    $self->{RESULT_SET}->addSelect($value);
}
sub maybeHost {
    my ($self, $atom, $qp) = @_;
    if (exists $self->{HOST_symbols}{$qp->toString}) {
	my $ret = $self->{HOST_symbols}{$atom};
	if (!$ret) {
	    $ret = $self->{-atomDictionary}->getUriName("host_$self->{HOST_index}", $self->{-sourceAttribution}->getSource());
	    $self->{HOST_index}++;
	    # print "changed ".$atom->toString." to ".$ret->toString."\n";
	    $self->{HOST_symbols}{$atom} = $ret;
	    $atom = $ret;
	}
    }
    return $atom;
}
# </ACTION Acessors>

# <Database sources-related stuff>
sub addSource {
    my ($self, $name, $source) = @_;
    $self->{-sources}{$name ? $name->toString : ''} = $source;
}
sub getDefaultDB {$_[0]->{-rdfDB}}
sub getTemplateDB {
    my ($self) = @_;
    if (!$self->{-templateDB}) {
	$self->{-templateDB} = new W3C::Rdf::RdfDB(-atomDictionary => $self->{-atomDictionary});
    }
    return $self->{-templateDB};
}
sub getSourceByName {$_[0]->{-sources}{$_[1] ? $_[1]->toString : ''} || &throw(new W3C::Util::Exception(-message => (ref $_[1]).' '.$_[1]->toString.' database not known'))}
sub getSourceAttribution {$_[0]->{-sourceAttribution} ? 
			      $_[0]->{-sourceAttribution} : 
			      &throw(new W3C::Util::Exception(-message => 
							      "need an attribution -- where did these statements come from?"))}
sub setSourceAttribution {$_[0]->{-sourceAttribution} = $_[1]}
# </Database sources-related stuff>

# <QUERY>
sub getReport {
    my ($self) = @_;
    my ($nodes, $statements) = $self->{RESULT_SET}->gatherCollect;
    return ($nodes, $self->{LABELS}, [], $statements);
}

# </QUERY>

1;

__END__

=head1 NAME

W3C::Rdf::Algae2 - a parser/evaluator for the algae2 language

=head1 SYNOPSIS

  use W3C::Rdf::Atoms;
  use W3C::Rdf::RdfDB;
  use W3C::Util::NamespaceHandler;
  use W3C::Rdf::Algae2;

  my $atoms = new W3C::Rdf::Atoms();
  my $db = new W3C::Rdf::RdfDB(-atomDictionary => $atoms);
  my $nsh = new W3C::Util::NamespaceHandler();
  my $url = $atoms->getUri(new_abs URI($0, 'file://localhost/some/path'));
  $attrib = $atoms->getGroundFactAttribution($url, undef, undef, undef);
  my $queryHandler = new W3C::Rdf::Algae2($atoms
					  $nsh, 
					  {'' => $self}, $self, 
					  $attrib, 
					  {-uniqueResults => 1}, 
					  -rdfDB => $db);
  my ($nodes, $selects, $messages, $proofs) = 
      $queryHandler->interpret('ask (...)', $self->{-location}, $QL_ALGAE);

=head1 DESCRIPTION

The Algae2 modules uses a parser to generated an AlgaeCompileTree. Algae2
calls the compile tree to perforn queries/assertions/rules.

This module is part of the W3C::Rdf CPAN module.

=head1 METHODS

=head2 new

    $atomDictionary, $namespaceHandler, 
	$sources, $rdfApp, $sourceAttrib, $flags

returns a new Algae2 object

=head2 interpret

    $algaeString, $location, $lang

returns: a list with the following elements

  1 array of tuples of bindings
  2 list of selected variables
  3 list of human-readable messages that occured during processing
  4 array of RdfDBs with the supporting statements for each tuple

=head1 AUTHOR

Eric Prud'hommeaux <eric@w3.org>

=head1 SEE ALSO

L<W3C::Rdf::RdfDB>
L<W3C::Rdf::AlgaeCompileTree>

=cut
